#include <stdlib.h>
#include "argon.h"
#include "random.h"

static void fill_r(t_state *state, t_parameters *pars)
{
	int n=pars->n;

	t_coord b[3]={{1, 0, 0, 0}, {0.5, sqrt(3)*0.5, 0, 0}, {0.5, sqrt(3)/6.0, sqrt(2.0/3.0), 0}};

	for(int i0=0; i0<n; i0++)
		for(int i1=0; i1<n; i1++)
			for(int i2=0; i2<n; i2++) {
				int i=i0+i1*n+i2*n*n;

				float offset=(n-1)*0.5;

				state->r[i].x=pars->a*((i0-offset)*b[0].x+(i1-offset)*b[1].x+(i2-offset)*b[2].x);
				state->r[i].y=pars->a*((i0-offset)*b[0].y+(i1-offset)*b[1].y+(i2-offset)*b[2].y);
				state->r[i].z=pars->a*((i0-offset)*b[0].z+(i1-offset)*b[1].z+(i2-offset)*b[2].z);
			}
}

static void fill_E_kin(t_parameters *pars; t_coord E_kin[pars->N], t_parameters *pars)
{
	int N=pars->N;

	t_coord E_sum={0,0,0,0};

	for(int i=1; i<N; i++) {
		E_kin[i].x=-0.5*K_BOLTZMANN*pars->T_0*log(uni_ran_01());
		E_kin[i].y=-0.5*K_BOLTZMANN*pars->T_0*log(uni_ran_01());
		E_kin[i].z=-0.5*K_BOLTZMANN*pars->T_0*log(uni_ran_01());

		E_sum.x+=E_kin[i].x;
		E_sum.y+=E_kin[i].y;
		E_sum.z+=E_kin[i].z;

	}

	t_coord factor;

	E_sum.x/=N;
	E_sum.y/=N;
	E_sum.z/=N;

	double E=0.5*K_BOLTZMANN*pars->T_0;

	if(E_sum.x==0.0 || E_sum.y==0.0 || E_sum.z==0.0) {
		if(E!=0.0) abort();
		return;
	}

	factor.x=E/E_sum.x;
	factor.y=E/E_sum.y;
	factor.z=E/E_sum.z;

	for(int i=0; i<N; i++) {
		E_kin[i].x*=factor.x;
		E_kin[i].y*=factor.y;
		E_kin[i].z*=factor.z;
	}
}

static void fill_p(t_parameters *pars; t_state *state, t_coord E_kin[pars->N], t_parameters *pars)
{
	int N=pars->N;

	t_coord p_sum={0, 0, 0};

	for(int i=0; i<N; i++) {
	
		state->p[i].x=sign_ran()*sqrt(2*pars->m*E_kin[i].x);
		state->p[i].y=sign_ran()*sqrt(2*pars->m*E_kin[i].y);
		state->p[i].z=sign_ran()*sqrt(2*pars->m*E_kin[i].z);

		p_sum.x+=state->p[i].x;
		p_sum.y+=state->p[i].y;
		p_sum.z+=state->p[i].z;
	}


	p_sum.x/=N;
	p_sum.y/=N;
	p_sum.z/=N;

	for(int i=0; i<N; i++) {
		state->p[i].x-=p_sum.x;
		state->p[i].y-=p_sum.y;
		state->p[i].z-=p_sum.z;
	}
}


void init_state(t_state *state, t_parameters *pars)
{
	int N=pars->N;

	fill_r(state, pars);

	t_coord E_kin[N];
	memset(E_kin, 0, sizeof(E_kin));
	fill_E_kin(E_kin, pars);
	fill_p(state, E_kin, pars);
}
